import fs from 'fs';
import path from 'path';
import prompts from 'prompts';
import { yellow, blue, green, cyan, reset, underline, red } from 'kolorist';
import { fileURLToPath } from 'url';
import { createSpinner } from 'nanospinner';
import { exec } from 'child_process';

const FRAMEWORKS = [
    {
        name: 'vanilla',
        display: 'Vanilla',
        color: yellow,
        variants: [
            {
                name: 'vanilla',
                display: 'JavaScript',
                color: yellow
            },
            {
                name: 'vanilla-ts',
                display: 'TypeScript',
                color: blue
            }
        ]
    },
    {
        name: 'vue',
        display: 'Vue',
        color: green,
        variants: [
            {
                name: 'vue',
                display: 'JavaScript',
                color: yellow
            },
            {
                name: 'vue-ts',
                display: 'TypeScript',
                color: blue
            }
        ]
    },
    {
        name: 'react',
        display: 'React',
        color: cyan,
        variants: [
            {
                name: 'react',
                display: 'JavaScript',
                color: yellow
            },
            {
                name: 'react-ts',
                display: 'TypeScript',
                color: blue
            }
        ]
    }
];
FRAMEWORKS.map((f) => (f.variants && f.variants.map((v) => v.name)) || [f.name]).reduce((a, b) => a.concat(b), []);

const root = process.cwd();
function pkgFromUserAgent(userAgent) {
    if (!userAgent)
        return undefined;
    const pkgSpec = userAgent.split(' ')[0];
    const pkgSpecArr = pkgSpec.split('/');
    return {
        name: pkgSpecArr[0],
        version: pkgSpecArr[1]
    };
}
const getPackageManager = () => {
    const pkgInfo = pkgFromUserAgent(process.env.npm_config_user_agent);
    return pkgInfo ? pkgInfo.name : 'npm';
};
function copy(src, dest) {
    const stat = fs.statSync(src);
    if (stat.isDirectory()) {
        copyDir(src, dest);
    }
    else {
        fs.copyFileSync(src, dest);
    }
}
function copyDir(srcDir, destDir) {
    fs.mkdirSync(destDir, { recursive: true });
    for (const file of fs.readdirSync(srcDir)) {
        const srcFile = path.resolve(srcDir, file);
        const destFile = path.resolve(destDir, file);
        copy(srcFile, destFile);
    }
}
const askForProjectLint = async () => {
    let result;
    try {
        result = await prompts([
            {
                type: 'select',
                name: 'framework',
                message: reset('Select a framework:'),
                initial: 0,
                choices: FRAMEWORKS.map((framework) => {
                    const frameworkColor = framework.color;
                    return {
                        title: frameworkColor(framework.display || framework.name),
                        value: framework
                    };
                })
            },
            {
                type: (framework) => (framework && framework.variants ? 'select' : null),
                name: 'variant',
                message: reset('Select a variant:'),
                choices: (framework) => framework.variants.map((variant) => {
                    const variantColor = variant.color;
                    return {
                        title: variantColor(variant.display || variant.name),
                        value: variant.name
                    };
                })
            },
            {
                type: 'multiselect',
                name: 'otherLint',
                message: underline('Pick other lint'),
                choices: [
                    { title: 'commitlint', value: 'commitlint' },
                    { title: 'stylelint', value: 'stylelint' }
                ],
                hint: '- Space to select. Return to submit'
            },
            {
                type: 'confirm',
                name: 'isVscode',
                message: underline('loading default vscode setting?'),
                initial: false
            }
        ], {
            onCancel: () => {
                throw new Error(red('✖') + ' Operation cancelled');
            }
        });
        return result;
    }
    catch (cancelled) {
        console.log(cancelled.message);
        return;
    }
};
const write = (file, content) => {
    const targetPath = path.join(root, file);
    if (content) {
        fs.writeFileSync(targetPath, content);
    }
    else {
        copy(path.join(root, file), targetPath);
    }
};
// TODO: add husky config and lint-stage config
const generatePackageJson = ({ otherLint, variant }) => {
    const huskyConfig = {
        hooks: {
            'pre-commit': 'lint-staged'
        }
    };
    const lintStagedConfig = {
        '*.{js,jsx,ts,tsx}': ['eslint --fix', 'prettier --write'],
        '{!(package)*.json,*.code-snippets,.!(browserslist)*rc}': ['prettier --write--parser json'],
        'package.json': ['prettier --write'],
        '*.md': ['prettier --write']
    };
    let scripts = {
        'lint:eslint': 'eslint --cache --max-warnings 0  "{src,mock}/**/*.{vue,ts,tsx}" --fix',
        'lint:prettier': 'prettier --write  "src/**/*.{js,json,tsx,css,less,scss,vue,html,md}"',
        'lint:lint-staged': 'lint-staged',
        prepare: 'husky install'
    };
    if (otherLint.includes('stylelint')) {
        lintStagedConfig['*.{scss,less,styl}'] = ['stylelint --fix', 'prettier --write'];
        scripts['lint:stylelint'] =
            'stylelint --cache --fix "**/*.{vue,less,postcss,css,scss}" --cache --cache-location node_modules/.cache/stylelint/';
    }
    if (variant.includes('vue')) {
        if (otherLint.includes('stylelint')) {
            lintStagedConfig['*.vue'] = ['eslint --fix', 'prettier --write', 'stylelint --fix'];
        }
        else {
            lintStagedConfig['*.vue'] = ['eslint --fix', 'prettier --write'];
        }
    }
    const pkg = JSON.parse(fs.readFileSync(path.join(root, `package.json`), 'utf-8'));
    pkg.scripts = {
        ...pkg.scripts,
        ...scripts
    };
    pkg['lint-staged'] = {
        ...pkg['lint-staged'],
        ...lintStagedConfig
    };
    pkg['husky'] = {
        ...pkg['husky'],
        ...huskyConfig
    };
    write('package.json', JSON.stringify(pkg, null, 2));
};

const commonPackages = [
    'husky',
    'lint-staged',
    'eslint',
    'prettier',
    'eslint-plugin-prettier',
    'eslint-config-prettier'
];
const commitlintPackages = [
    '@commitlint/cli',
    '@commitlint/config-conventional',
    'commitizen',
    'commitlint-config-cz'
];
const stylelintPackages = [
    'stylelint',
    'stylelint-config-prettier',
    'stylelint-config-rational-order',
    'stylelint-config-standard',
    'stylelint-order'
];
const eslintConfig = {
    env: {
        browser: true,
        es2021: true,
        node: true
    },
    overrides: []
};
const eslintOverrides = {
    vanilla: [
        {
            files: ['*.js'],
            extends: ['eslint:recommended', 'plugin:prettier/recommended']
        }
    ],
    'vanilla-ts': [
        {
            files: ['*.js'],
            extends: ['eslint:recommended', 'plugin:prettier/recommended']
        },
        {
            files: ['*.ts'],
            parser: '@typescript-eslint/parser',
            parserOptions: {
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: [
                'eslint:recommended',
                'plugin:@typescript-eslint/recommended',
                'plugin:prettier/recommended'
            ]
        }
    ],
    vue: [
        {
            files: ['*.js'],
            extends: ['eslint:recommended', 'plugin:prettier/recommended']
        },
        {
            files: ['*.vue'],
            parser: 'vue-eslint-parser',
            parserOptions: {
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: ['eslint:recommended', 'plugin:vue/vue3-recommended', 'plugin:prettier/recommended'],
            rules: {
                'vue/multi-word-component-names': 'off'
            }
        }
    ],
    'vue-ts': [
        {
            files: ['*.js'],
            extends: ['eslint:recommended', 'plugin:prettier/recommended']
        },
        {
            files: ['*.ts'],
            parser: '@typescript-eslint/parser',
            parserOptions: {
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: [
                'eslint:recommended',
                'plugin:@typescript-eslint/recommended',
                'plugin:prettier/recommended'
            ]
        },
        {
            files: ['*.vue'],
            parser: 'vue-eslint-parser',
            parserOptions: {
                parser: '@typescript-eslint/parser',
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: [
                'eslint:recommended',
                'plugin:@typescript-eslint/recommended',
                'plugin:vue/vue3-recommended',
                'plugin:prettier/recommended'
            ],
            rules: {
                'vue/multi-word-component-names': 'off'
            }
        }
    ],
    react: [
        {
            files: ['*.js', '*.jsx'],
            parserOptions: {
                ecmaFeatures: {
                    jsx: true
                },
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: ['eslint:recommended', 'plugin:react/recommended', 'plugin:prettier/recommended'],
            rules: {
                'react/react-in-jsx-scope': 'off'
            }
        }
    ],
    'react-ts': [
        {
            files: ['*.js'],
            extends: ['eslint:recommended', 'plugin:prettier/recommended']
        },
        {
            files: ['*.ts', '*.tsx'],
            parser: '@typescript-eslint/parser',
            parserOptions: {
                ecmaFeatures: {
                    jsx: true
                },
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: [
                'eslint:recommended',
                'plugin:@typescript-eslint/recommended',
                'plugin:react/recommended',
                'plugin:prettier/recommended'
            ],
            rules: {
                'react/react-in-jsx-scope': 'off'
            }
        }
    ]
};
const eslintPackages = {
    vanilla: [],
    'vanilla-ts': ['@typescript-eslint/eslint-plugin', '@typescript-eslint/parser'],
    vue: ['vue-eslint-parser', 'eslint-plugin-vue'],
    'vue-ts': [
        '@typescript-eslint/eslint-plugin',
        '@typescript-eslint/parser',
        'vue-eslint-parser',
        'eslint-plugin-vue'
    ],
    react: ['eslint-plugin-react'],
    'react-ts': [
        '@typescript-eslint/eslint-plugin',
        '@typescript-eslint/parser',
        'eslint-plugin-react'
    ]
};

const writeTemplateFile = (otherLint, isVscode) => {
    const templateDir = path.resolve(fileURLToPath(import.meta.url), '../../template');
    const files = fs.readdirSync(templateDir);
    let filterResult = files;
    if (otherLint.indexOf('stylelint') === -1) {
        filterResult = filterResult.filter((file) => file !== '.stylelintignore' && file !== '.stylelintrc.json');
    }
    if (otherLint.indexOf('commitlint') === -1) {
        filterResult = filterResult.filter((file) => file !== '.cz-config.cjs' && file !== '.commitlintrc.json');
    }
    if (!isVscode) {
        filterResult = filterResult.filter((file) => file !== '.vscode');
    }
    for (const file of filterResult) {
        const targetPath = path.join(root, file);
        copy(path.join(templateDir, file), targetPath);
    }
};
const writeEslintFile = (eslint) => {
    const eslintFile = path.join(root, '.eslintrc.json');
    fs.writeFileSync(eslintFile, JSON.stringify(eslint, null, 2));
};
const execHuskyCommand = (otherLint) => {
    const initGit = 'git init';
    const initHusky = 'npx husky install';
    const preCommit = `npx husky add .husky/pre-commit "npm run lint:lint-staged"`;
    if (!fs.existsSync(path.join(root, '.git'))) {
        exec(initGit, { cwd: root }, (error) => {
            if (error) {
                console.error(error);
                return;
            }
        });
    }
    exec(initHusky, { cwd: root }, (error) => {
        if (error) {
            console.error(error);
            return;
        }
        exec(preCommit, { cwd: root }, (error) => {
            if (error) {
                console.error(error);
                return;
            }
        });
        if (otherLint.includes('commitlint')) {
            const commitMsg = `npx husky add .husky/commit-msg 'npx --no-install commitlint --edit "$1"'`;
            exec(commitMsg, { cwd: root }, (error) => {
                if (error) {
                    console.error(error);
                    return;
                }
            });
        }
    });
};
const getPackageList = ({ otherLint, variant }) => {
    let result = [];
    if (otherLint.includes('stylelint')) {
        result = [...result, ...stylelintPackages];
    }
    if (otherLint.includes('commitlint')) {
        result = [...result, ...commitlintPackages];
    }
    return [...commonPackages, ...eslintPackages[variant], ...result];
};
const settingLint = ({ otherLint, variant, isVscode }) => {
    const packageManager = getPackageManager();
    const eslint = { ...eslintConfig, overrides: eslintOverrides[variant] };
    const packageList = getPackageList({ otherLint, variant });
    generatePackageJson({ otherLint, variant });
    const commandMap = {
        npm: `npm install --save-dev ${packageList.join(' ')}`,
        yarn: `yarn add --dev ${packageList.join(' ')}`,
        pnpm: `pnpm install --save-dev ${packageList.join(' ')}`
    };
    const installCommand = commandMap[packageManager];
    const spinner = createSpinner('Installing packages...').start();
    exec(installCommand, { cwd: root }, (error) => {
        if (error) {
            spinner.error({
                text: red('Failed to install packages!'),
                mark: '✖'
            });
            console.error(error);
            return;
        }
        execHuskyCommand(otherLint);
        writeTemplateFile(otherLint, isVscode);
        writeEslintFile(eslint);
        spinner.success({ text: green('All done! 🎉'), mark: '✔' });
        console.log(cyan('\n🔥 Reload your editor to activate the settings!'));
    });
};

async function init() {
    const result = await askForProjectLint();
    console.log(cyan('will integrates the default with husky and lint-stage'));
    settingLint(result);
}
init();

if(typeof window !== 'undefined') {
  window._VERSION_ = '1.0.3'
}
